%%% Script to estimate manifold signal components from
%%% compressive measurements (1D case)
%%% Chin, March 2012

%%% Paper: "Signal Recovery on Incoherent Manifolds"
%%% Authors: Chinmay Hegde and Richard G. Baraniuk
%%% Submitted to IEEE Trans. Info. Theory, Jan 2012

clear
close all
clc

N = 10000; sgm = 5; wndw = 50;
t = linspace(-wndw,wndw,N)';
T0 = random('unid',25); %%% 
sigman = 0*0.005; %measurement noise


%---original template (zero-centered Gaussian)
x0 = exp(-(t/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
x0 = x0(:);

% signal component 1 (shifted Gaussian)
x = exp(-((t-T0)/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
x1 = x(:);


% signal component 2 (sparse noise)
K = 10;
x2 = 0*x1; v = randperm(length(x1));
x2(v(1:K)) = max(x0(:)) + max(x0(:))*randn(K,1);


y = x1 + x2;

M = 200;
Phi = (1/sqrt(M))*randn(M,N); %Phi = Phi/norm(Phi(:));
n = sigman*randn(M,1);
z = Phi*(y)+n;

recopts.stp = .5; %%% keep this below 0.7

y1 = y;
recopts.iter = 300;
recopts.base = x0;
recopts.orig = y1;
recopts.K = K;
recopts.init = 0*circshift(x0,0);
recopts.maxshft = 25;

[yhat, ahat, bhat] = spinrec_1d(z,Phi,recopts);

ax = [0 10000 -0.1 0.2]; %%% set axis for plotting

%close
%figure, plot(y), hold on, plot(yhat,'r'), 
disp(norm(y1-yhat)/norm(y1))
figure, plot(y, 'LineWidth', 2);
axisfortex('','','')
grid on
axis(ax)

figure, plot(ahat, 'r', 'LineWidth', 2);
axisfortex('','','')
grid on
axis(ax)

figure, plot(bhat, 'r', 'LineWidth', 2);
axisfortex('','','')
grid on
axis(ax)
