%%% Script to estimate manifold signal components from
%%% noisy compressive measurements (2D case)
%%% Chin, March 2012

%%% Paper: "Signal Recovery on Incoherent Manifolds"
%%% Authors: Chinmay Hegde and Richard G. Baraniuk
%%% Submitted to IEEE Trans. Info. Theory, Jan 2012


clear
close all
clc

addpath data/

load trans_man_new %%% centered large circle
recopts.N = 64; sigman = 0;
recopts.base1 = reshape(tr_disk(:,181),[],64);

load trans_man %%% centered small square
recopts.base2 = reshape(tr_sq(:,365),[],64);


recopts.init = recopts.base1(:) + recopts.base2(:);

T1 = random('unid',15,[1 2]);
im1 = circshift(recopts.base1,T1);
T2 = random('unid',15,[1 2]);
im2 = circshift(recopts.base2,T2);

im_noise = .05*randn(size(im1)); %%% signal noise
x1 = im1(:); x2 = im2(:); n1 = im_noise(:);
y = x1 + x2 + n1; %%% noisy measurement vector
N = length(y);

M = 50;
Phi = (1/sqrt(M))*randn(M,N); %Phi = Phi/norm(Phi(:));
n = sigman*randn(M,1);
z = Phi*y + n;

recopts.stp = .5;
recopts.iter = 100;
recopts.orig = y;
recopts.maxshft = 25;

[yhat, ahat, bhat] = spinrec_2d(z,Phi,recopts);

disp(norm(y-yhat)/norm(y))

close all
figure, imagesc(im1 + im2 + im_noise), axis image, box on;  rmaxis, colormap(gray), brighten(.6)
figure, imagesc(reshape(ahat,64,[])), axis image, box on; rmaxis, colormap(gray)
figure, imagesc(reshape(bhat,64,[])), axis image,  box on; rmaxis, colormap(gray)