%%% Script to generate a Monte Carlo plot of number of measurements
%%% M vs reconstruction SNR in dB
%%% Chin, March 2012

%%% Paper: "Signal Recovery on Incoherent Manifolds"
%%% Authors: Chinmay Hegde and Richard G. Baraniuk
%%% Submitted to IEEE Trans. Info. Theory, Jan 2012



%clear
close all
clc

N = 10000; sgm = 5; wndw = 50;
t = linspace(-wndw,wndw,N)';
MM = 20:20:200; E = 100;

%---original template
x0 = exp(-(t/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
x0 = x0(:);

K = 5;
snr1 = zeros(E,length(MM));
for ee=1:E
    disp(ee)
    mm=1;
    for M = MM

        T0 = random('unid',25);
        sigman = 0*0.005; %%% measurement noise
        % signal component 1
        x = exp(-((t-T0)/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
        x1 = x(:);
        % sparse noise
        x2 = 0*x1; v = randperm(length(x1));
        x2(v(1:K)) = max(x0(:)) + max(x0(:))*randn(K,1);
        
        y = x1 + x2;
        Phi = (1/sqrt(M))*randn(M,N); 
        n = sigman*randn(M,1);
        z = Phi*(y)+n;
        
        recopts.stp = .5; %%% keep this less than 0.7
        
        y1 = y;
        recopts.iter = 100;
        recopts.base = x0;
        recopts.orig = y1;
        recopts.K = K;
        recopts.init = 0*circshift(x0,0);
        recopts.maxshft = 25;
        [yhat, ahat, bhat] = spinrec_1d(z,Phi,recopts);
        
        mse = norm(x1-ahat)/norm(x1);
        snr1(ee,mm) = -20*log10(mse);
        mm = mm+1;
    end
end
snr_avg1 = mean(snr1,1);

K = 10;
snr2 = zeros(E,length(MM));
for ee=1:E
    disp(ee)
    mm=1;
    for M = MM

        T0 = random('unid',25);
        sigman = 0*0.005; %%% measurement noise
        % signal component 1
        x = exp(-((t-T0)/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
        x1 = x(:);
        % sparse noise
        x2 = 0*x1; v = randperm(length(x1));
        x2(v(1:K)) = max(x0(:)) + max(x0(:))*randn(K,1);
        
        y = x1 + x2;
        Phi = (1/sqrt(M))*randn(M,N); %Phi = Phi/norm(Phi(:));
        n = sigman*randn(M,1);
        z = Phi*(y)+n;
        
        recopts.stp = .5;  %%% keep this less than 0.7
        
        y1 = y;
        recopts.iter = 100;
        recopts.base = x0;
        recopts.orig = y1;
        recopts.K = K;
        recopts.init = 0*circshift(x0,0);
        recopts.maxshft = 25;
        [yhat, ahat, bhat] = spinrec_1d(z,Phi,recopts);
        
        mse = norm(x1-ahat)/norm(x1);
        snr2(ee,mm) = -20*log10(mse);
        mm = mm+1;
    end
end
snr_avg2 = mean(snr2,1);

K = 15;
snr3 = zeros(E,length(MM));
for ee=1:E
    disp(ee)
    mm=1;
    for M = MM

        T0 = random('unid',25);
        sigman = 0*0.005; %%% measurement noise
        % signal component 1
        x = exp(-((t-T0)/(sqrt(2)*sgm)).^2)/sqrt(2*pi*sgm^2); 
        x1 = x(:);
        % sparse noise
        x2 = 0*x1; v = randperm(length(x1));
        x2(v(1:K)) = max(x0(:)) + max(x0(:))*randn(K,1);
        
        y = x1 + x2;
        Phi = (1/sqrt(M))*randn(M,N); %Phi = Phi/norm(Phi(:));
        n = sigman*randn(M,1);
        z = Phi*(y)+n;
        
        recopts.stp = .5;  %%% keep this less than 0.7
        
        y1 = y;
        recopts.iter = 100;
        recopts.base = x0;
        recopts.orig = y1; 
        recopts.K = K;
        recopts.init = 0*circshift(x0,0);
        recopts.maxshft = 25;
        [yhat, ahat, bhat] = spinrec_1d(z,Phi,recopts);
        
        mse = norm(x1-ahat)/norm(x1);
        snr3(ee,mm) = -20*log10(mse);
        mm = mm+1;
    end
end
snr_avg3 = mean(snr3,1);

figure(1), hold on
plot(MM,snr_avg1,'b*-','LineWidth',2)
plot(MM,snr_avg2,'kd--','LineWidth',2)
plot(MM,snr_avg3,'ro-.','LineWidth',2)
box on
axisfortex('','Number of measurements M','Reconstruction SNR (dB)')
grid on
axis tight
legend('K = 5','K = 10','K = 15','Location','NorthWest')