%%% Function to estimate manifold signal components from
%%% compressive measurements (2D case)
%%% Chin, March 2012

%%% Paper: "Signal Recovery on Incoherent Manifolds"
%%% Authors: Chinmay Hegde and Richard G. Baraniuk
%%% Submitted to IEEE Trans. Info. Theory, Jan 2012

%%% INPUTS: y: Mx1 measurement vector
%%%         Phi: MxN measurement matrix
%%%         recopts: additional parameters
%%% OUTPUTS: xhat, ahat, bhat: signal estimates.
%%%          xhat = ahat + bhat
%%% SUBROUTINES: proj2: performs 2D matched filtering

function [xhat,ahat,bhat] = spinrec_2d(y, Phi,recopts)

%%% 2 translating templates

stp = recopts.stp;
iter = recopts.iter;
x1 = recopts.base1; 
x2 = recopts.base2;
xhat = recopts.init;

ahat = 0*xhat; bhat = 0*xhat;
xhat = ahat + bhat;
xhat_prev = xhat;

%K = recopts.K;

converr = 1;
kk = 1;

while kk < iter && converr > 1e-10
    
    disp([kk converr])
   % figure(1), clf, plot(recopts.orig,'r')
   % hold on, plot(xhat,'k'), drawnow, pause(0.5)
    
    r = y - Phi*xhat_prev;
    
    s1 = ahat + stp*(Phi'*r);
    s2 = bhat + stp*(Phi'*r);
    
    [ahat,~] = proj2(s1,x1,recopts);
    [bhat,~] = proj2(s2,x2,recopts);
    
    xhat = ahat + bhat;
    
    kk = kk+1;
    converr = norm(xhat-xhat_prev)/norm(xhat);
    xhat_prev = xhat;

end

end



% manifold projection function (match filter with template x0)

function [xhat,imhat] = proj2(z,x0,recopts)

%maxshft = recopts.maxshft;
% find the best shift
N = recopts.N;
im0 = reshape(x0,[],N);
im = reshape(z,[],N);

c = xcorr2(im,im0);
[i,j] = find(c == max(c(:)));

T0 = [i,j] - [N,N];
imhat = circshift(im0,T0);

xhat = imhat(:);

end